@@ -1,8 +1,13 @@
 # showstack.py - extension to dump a Python stack trace on signal
 #
 # binds to both SIGQUIT (Ctrl-\) and SIGINFO (Ctrl-T on BSDs)
+"""dump stack trace when receiving SIGQUIT (Ctrl-\) and SIGINFO (Ctrl-T on BSDs)
+"""
 
-import sys, signal, traceback
+from __future__ import absolute_import
+import signal
+import sys
+import traceback
 
 def sigshow(*args):
     sys.stderr.write("\n")
