@@ -5,14 +5,23 @@
 #
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
+from __future__ import absolute_import
+
+import email.parser as emailparser
+import os
+import shutil
+import stat
+import tempfile
 
-from common import NoRepo, commandline, commit, converter_source
 from mercurial.i18n import _
-from mercurial import encoding, util
-import os, shutil, tempfile, stat
-from email.Parser import Parser
+from mercurial import (
+    encoding,
+    error,
+    util,
+)
+from . import common
 
-class gnuarch_source(converter_source, commandline):
+class gnuarch_source(common.converter_source, common.commandline):
 
     class gnuarch_rev(object):
         def __init__(self, rev):
@@ -27,11 +36,11 @@ class gnuarch_source(converter_source, commandline):
             self.ren_files = {}
             self.ren_dirs = {}
 
-    def __init__(self, ui, path, revs=None):
-        super(gnuarch_source, self).__init__(ui, path, revs=revs)
+    def __init__(self, ui, repotype, path, revs=None):
+        super(gnuarch_source, self).__init__(ui, repotype, path, revs=revs)
 
         if not os.path.exists(os.path.join(path, '{arch}')):
-            raise NoRepo(_("%s does not look like a GNU Arch repository")
+            raise common.NoRepo(_("%s does not look like a GNU Arch repository")
                          % path)
 
         # Could use checktool, but we want to check for baz or tla.
@@ -42,9 +51,9 @@ class gnuarch_source(converter_source, commandline):
             if util.findexe('tla'):
                 self.execmd = 'tla'
             else:
-                raise util.Abort(_('cannot find a GNU Arch tool'))
+                raise error.Abort(_('cannot find a GNU Arch tool'))
 
-        commandline.__init__(self, ui, self.execmd)
+        common.commandline.__init__(self, ui, self.execmd)
 
         self.path = os.path.realpath(path)
         self.tmppath = None
@@ -54,7 +63,7 @@ class gnuarch_source(converter_source, commandline):
         self.changes = {}
         self.parents = {}
         self.tags = {}
-        self.catlogparser = Parser()
+        self.catlogparser = emailparser.Parser()
         self.encoding = encoding.encoding
         self.archives = []
 
@@ -135,7 +144,7 @@ class gnuarch_source(converter_source, commandline):
 
     def getfile(self, name, rev):
         if rev != self.lastrev:
-            raise util.Abort(_('internal calling inconsistency'))
+            raise error.Abort(_('internal calling inconsistency'))
 
         if not os.path.lexists(os.path.join(self.tmppath, name)):
             return None, None
@@ -144,7 +153,7 @@ class gnuarch_source(converter_source, commandline):
 
     def getchanges(self, rev, full):
         if full:
-            raise util.Abort(_("convert from arch do not support --full"))
+            raise error.Abort(_("convert from arch does not support --full"))
         self._update(rev)
         changes = []
         copies = {}
@@ -175,8 +184,9 @@ class gnuarch_source(converter_source, commandline):
 
     def getcommit(self, rev):
         changes = self.changes[rev]
-        return commit(author=changes.author, date=changes.date,
-                      desc=changes.summary, parents=self.parents[rev], rev=rev)
+        return common.commit(author=changes.author, date=changes.date,
+                             desc=changes.summary, parents=self.parents[rev],
+                             rev=rev)
 
     def gettags(self):
         return self.tags
@@ -287,7 +297,7 @@ class gnuarch_source(converter_source, commandline):
                 self.changes[rev].continuationof = self.recode(
                     catlog['Continuation-of'])
         except Exception:
-            raise util.Abort(_('could not parse cat-log of %s') % rev)
+            raise error.Abort(_('could not parse cat-log of %s') % rev)
 
     def _parsechangeset(self, data, rev):
         for l in data:
