@@ -8,19 +8,29 @@
 '''command to view revision graphs from a shell (DEPRECATED)
 
 The functionality of this extension has been include in core Mercurial
-since version 2.3.
+since version 2.3. Please use :hg:`log -G ...` instead.
 
 This extension adds a --graph option to the incoming, outgoing and log
 commands. When this options is given, an ASCII representation of the
 revision graph is also shown.
 '''
 
+from __future__ import absolute_import
+
 from mercurial.i18n import _
-from mercurial import cmdutil, commands
+from mercurial import (
+    cmdutil,
+    commands,
+    registrar,
+)
 
 cmdtable = {}
-command = cmdutil.command(cmdtable)
-testedwith = 'internal'
+command = registrar.command(cmdtable)
+# Note for extension authors: ONLY specify testedwith = 'ships-with-hg-core' for
+# extensions which SHIP WITH MERCURIAL. Non-mainline extensions should
+# be specifying the version(s) of Mercurial they are tested with, or
+# leave the attribute unspecified.
+testedwith = 'ships-with-hg-core'
 
 @command('glog',
     [('f', 'follow', None,
@@ -42,10 +52,10 @@ testedwith = 'internal'
      _('show changesets within the given named branch'), _('BRANCH')),
     ('P', 'prune', [],
      _('do not display revision or any of its ancestors'), _('REV')),
-    ] + commands.logopts + commands.walkopts,
+    ] + cmdutil.logopts + cmdutil.walkopts,
     _('[OPTION]... [FILE]'),
     inferrepo=True)
-def graphlog(ui, repo, *pats, **opts):
+def glog(ui, repo, *pats, **opts):
     """show revision history alongside an ASCII revision graph
 
     Print a revision history alongside a revision graph drawn with
@@ -53,6 +63,8 @@ def graphlog(ui, repo, *pats, **opts):
 
     Nodes printed as an @ character are parents of the working
     directory.
+
+    This is an alias to :hg:`log -G`.
     """
-    opts['graph'] = True
+    opts[r'graph'] = True
     return commands.log(ui, repo, *pats, **opts)
