@@ -18,7 +18,7 @@ from . import (
 
 _mdre = re.compile('\1\n')
 def parsemeta(text):
-    """return (metadatadict, keylist, metadatasize)"""
+    """return (metadatadict, metadatasize)"""
     # text can be buffer, so we can't use .startswith or .index
     if text[:2] != '\1\n':
         return None, None
@@ -31,7 +31,7 @@ def parsemeta(text):
     return meta, (s + 2)
 
 def packmeta(meta, text):
-    keys = sorted(meta.iterkeys())
+    keys = sorted(meta)
     metatext = "".join("%s: %s\n" % (k, meta[k]) for k in keys)
     return "\1\n%s\1\n%s" % (metatext, text)
 
@@ -43,6 +43,8 @@ class filelog(revlog.revlog):
     def __init__(self, opener, path):
         super(filelog, self).__init__(opener,
                         "/".join(("data", path + ".i")))
+        # full name of the user visible file, relative to the repository root
+        self.filename = path
 
     def read(self, node):
         t = self.revision(node)
