@@ -12,33 +12,33 @@ from mercurial import (
 cmdtable = {}
 command = registrar.command(cmdtable)
 
-@command('autodiff',
-    [('', 'git', '', 'git upgrade mode (yes/no/auto/warn/abort)')],
-    '[OPTION]... [FILE]...')
+@command(b'autodiff',
+    [(b'', b'git', b'', b'git upgrade mode (yes/no/auto/warn/abort)')],
+    b'[OPTION]... [FILE]...')
 def autodiff(ui, repo, *pats, **opts):
     diffopts = patch.difffeatureopts(ui, opts)
-    git = opts.get('git', 'no')
+    git = opts.get(b'git', b'no')
     brokenfiles = set()
     losedatafn = None
-    if git in ('yes', 'no'):
-        diffopts.git = git == 'yes'
+    if git in (b'yes', b'no'):
+        diffopts.git = git == b'yes'
         diffopts.upgrade = False
-    elif git == 'auto':
+    elif git == b'auto':
         diffopts.git = False
         diffopts.upgrade = True
-    elif git == 'warn':
+    elif git == b'warn':
         diffopts.git = False
         diffopts.upgrade = True
         def losedatafn(fn=None, **kwargs):
             brokenfiles.add(fn)
             return True
-    elif git == 'abort':
+    elif git == b'abort':
         diffopts.git = False
         diffopts.upgrade = True
         def losedatafn(fn=None, **kwargs):
-            raise error.Abort('losing data for %s' % fn)
+            raise error.Abort(b'losing data for %s' % fn)
     else:
-        raise error.Abort('--git must be yes, no or auto')
+        raise error.Abort(b'--git must be yes, no or auto')
 
     node1, node2 = scmutil.revpair(repo, [])
     m = scmutil.match(repo[node2], pats, opts)
@@ -47,4 +47,4 @@ def autodiff(ui, repo, *pats, **opts):
     for chunk in it:
         ui.write(chunk)
     for fn in sorted(brokenfiles):
-        ui.write(('data lost for: %s\n' % fn))
+        ui.write((b'data lost for: %s\n' % fn))
