@@ -33,10 +33,30 @@ from __future__ import absolute_import
 
 import socket
 
+from mercurial import(
+    registrar,
+)
+
 from mercurial.hgweb import (
     server,
 )
 
+configtable = {}
+configitem = registrar.configitem(configtable)
+
+configitem('badserver', 'closeafteraccept',
+    default=False,
+)
+configitem('badserver', 'closeafterrecvbytes',
+    default=0,
+)
+configitem('badserver', 'closeaftersendbytes',
+    default=0,
+)
+configitem('badserver', 'closebeforeaccept',
+    default=False,
+)
+
 # We can't adjust __class__ on a socket instance. So we define a proxy type.
 class socketproxy(object):
     __slots__ = (
@@ -256,9 +276,9 @@ def extsetup(ui):
         def process_request(self, socket, address):
             # Wrap socket in a proxy if we need to count bytes.
             closeafterrecvbytes = self._ui.configint('badserver',
-                                                     'closeafterrecvbytes', 0)
+                                                     'closeafterrecvbytes')
             closeaftersendbytes = self._ui.configint('badserver',
-                                                     'closeaftersendbytes', 0)
+                                                     'closeaftersendbytes')
 
             if closeafterrecvbytes or closeaftersendbytes:
                 socket = socketproxy(socket, self.errorlog,
