@@ -3,13 +3,14 @@
 run-test.t only checks positive matches and can not see warnings
 (both by design)
 """
-from __future__ import print_function
+from __future__ import absolute_import, print_function
 
-import os, re
+import doctest
+import os
+import re
 # this is hack to make sure no escape characters are inserted into the output
 if 'TERM' in os.environ:
     del os.environ['TERM']
-import doctest
 run_tests = __import__('run-tests')
 
 def prn(ex):
@@ -38,7 +39,8 @@ def lm(expected, output):
             and output.endswith(b'\n')), 'missing newline'
     assert not re.search(br'[^ \w\\/\r\n()*?]', expected + output), \
            b'single backslash or unknown char'
-    match = run_tests.TTest.linematch(expected, output)
+    test = run_tests.TTest(b'test-run-test.t', b'.', b'.')
+    match = test.linematch(expected, output)
     if isinstance(match, str):
         return 'special: ' + match
     elif isinstance(match, bytes):
@@ -52,6 +54,8 @@ def wintests():
     enable windows matching on any os
         >>> _osaltsep = os.altsep
         >>> os.altsep = True
+        >>> _osname = os.name
+        >>> os.name = 'nt'
 
     valid match on windows
         >>> lm(b'g/a*/d (glob)\n', b'g\\abc/d\n')
@@ -63,10 +67,13 @@ def wintests():
 
     missing glob
         >>> lm(b'/g/c/d/fg\n', b'\\g\\c\\d/fg\n')
-        'special: +glob'
+        True
+        >>> lm(b'/g/c/d/fg\n', b'\\g\\c\\d\\fg\r\n')
+        True
 
     restore os.altsep
         >>> os.altsep = _osaltsep
+        >>> os.name = _osname
     """
     pass
 
@@ -76,6 +83,8 @@ def otherostests():
     disable windows matching on any os
         >>> _osaltsep = os.altsep
         >>> os.altsep = False
+        >>> _osname = os.name
+        >>> os.name = 'nt'
 
     backslash does not match slash
         >>> lm(b'h/a* (glob)\n', b'h\\ab\n')
@@ -91,6 +100,7 @@ def otherostests():
 
     restore os.altsep
         >>> os.altsep = _osaltsep
+        >>> os.name = _osname
     """
     pass
 
