@@ -1,15 +1,17 @@
-#!/usr/bin/env python
+from __future__ import absolute_import, print_function
 
 import os
-from mercurial import ui
+from mercurial import (
+    ui as uimod,
+)
 
 hgrc = os.environ['HGRCPATH']
 f = open(hgrc)
 basehgrc = f.read()
 f.close()
 
-print '      hgrc settings    command line options      final result   '
-print '    quiet verbo debug   quiet verbo debug      quiet verbo debug'
+print('      hgrc settings    command line options      final result   ')
+print('    quiet verbo debug   quiet verbo debug      quiet verbo debug')
 
 for i in xrange(64):
     hgrc_quiet   = bool(i & 1<<0)
@@ -30,7 +32,7 @@ for i in xrange(64):
         f.write('debug = True\n')
     f.close()
 
-    u = ui.ui()
+    u = uimod.ui.load()
     if cmd_quiet or cmd_debug or cmd_verbose:
         u.setconfig('ui', 'quiet', str(bool(cmd_quiet)))
         u.setconfig('ui', 'verbose', str(bool(cmd_verbose)))
@@ -43,7 +45,7 @@ for i in xrange(64):
     elif u.verbose and u.quiet:
         check = ' +'
 
-    print ('%2d  %5s %5s %5s   %5s %5s %5s  ->  %5s %5s %5s%s'
+    print(('%2d  %5s %5s %5s   %5s %5s %5s  ->  %5s %5s %5s%s'
            % (i, hgrc_quiet, hgrc_verbose, hgrc_debug,
               cmd_quiet, cmd_verbose, cmd_debug,
-              u.quiet, u.verbose, u.debugflag, check))
+              u.quiet, u.verbose, u.debugflag, check)))
